@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2011-2021 by the Free Software Foundation, Inc.
+# Copyright (C) 2011-2022 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -31,7 +31,6 @@ from datetime import datetime
 from email import message_from_bytes, policy
 from email.utils import make_msgid, unquote
 from math import floor
-from string import printable
 from traceback import print_exc
 
 from django.conf import settings
@@ -132,17 +131,20 @@ class DbImporter(object):
     def _get_date(self, message, header, report_name):
         try:
             date = message.get(header)
-            if date:
-                assert all(char in printable for char in date)
-                assert date.datetime is not None
-        except (AssertionError, TypeError, ValueError) as e:
+            if date and not date.datetime:
+                if self.verbose:
+                    self.stderr.write(
+                        "Bad datetime in {} header in message {}{}.".format(
+                            header, unquote(message.get("message-id", 'n/a')),
+                            report_name))
+                return None
+        except (AttributeError, TypeError, ValueError) as e:
             if self.verbose:
                 self.stderr.write(
                     "Can't get {} header in message {}{}: {}.".format(
                         header, unquote(message.get("message-id", 'n/a')),
                         report_name, e))
             return None
-
         return date
 
     def _fix_mid(self, mid):
@@ -219,8 +221,11 @@ class DbImporter(object):
                 message.set_unixfrom(unixfrom)
             if message['message-id'] is None:
                 message['Message-ID'] = make_msgid('generated')
-            message.replace_header(
-                'Message-ID', self._fix_mid(message['message-id']))
+            if message['message-id'].strip() == '':
+                message.replace_header('Message-ID', make_msgid('generated'))
+            if message['message-id'] != self._fix_mid(message['message-id']):
+                message.replace_header(
+                    'Message-ID', self._fix_mid(message['message-id']))
             # Now insert the message
             try:
                 with transaction.atomic():
