@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2014-2021 by the Free Software Foundation, Inc.
+# Copyright (C) 2014-2022 by the Free Software Foundation, Inc.
 #
 # This file is part of HyperKitty.
 #
@@ -23,8 +23,10 @@
 import logging
 import os
 import re
+from email.generator import BytesGenerator
 from email.message import EmailMessage
 from email.utils import formataddr
+from io import BytesIO
 
 from django.conf import settings
 from django.db import IntegrityError, models
@@ -175,7 +177,8 @@ class Email(models.Model):
         header_date = self.date.astimezone(tz).replace(microsecond=0)
         # Date format: http://tools.ietf.org/html/rfc5322#section-3.3
         msg["Date"] = header_date.strftime("%a, %d %b %Y %H:%M:%S %z")
-        msg["Message-ID"] = "<%s>" % self.message_id
+        # Clean bogus characters from Message-ID.
+        msg["Message-ID"] = "<%s>" % re.sub(r'[\s<>()]', '', self.message_id)
         if self.in_reply_to:
             msg["In-Reply-To"] = "<%s>" % unfold(self.in_reply_to)
 
@@ -194,6 +197,20 @@ class Email(models.Model):
 
         return msg
 
+    def as_bytes(self):
+        """Similar to EmailMessage.as_bytes() but sets mangle_from_=True.
+
+        Serialize the current email into bytes to be added into an mbox.
+        """
+        msg = self.as_message()
+        # We have to call BytesGenerator here to set
+        # mangle_from_=True, which is set to 'False' by default in
+        # EmailMessage.as_bytes() method.
+        fd = BytesIO()
+        gen = BytesGenerator(fd, mangle_from_=True)
+        gen.flatten(msg, unixfrom=True)
+        return fd.getvalue()
+
     @property
     def display_fixed(self):
         return "@@" in self.content
@@ -314,7 +331,7 @@ class Attachment(models.Model):
 
     def get_content(self):
         folder = self._get_folder()
-        if folder is None:
+        if folder is None or self.content is not None:
             return bytes(self.content)
         filepath = os.path.join(folder, str(self.counter))
         if not os.path.exists(filepath):
