@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2014-2021 by the Free Software Foundation, Inc.
+# Copyright (C) 2014-2022 by the Free Software Foundation, Inc.
 #
 # This file is part of HyperKitty.
 #
@@ -66,7 +66,7 @@ class MailingList(models.Model):
     """
     An archived mailing-list.
     """
-    name = models.CharField(max_length=254, unique=True)
+    name = models.CharField(db_index=True, max_length=254, unique=True)
     list_id = models.CharField(max_length=254, null=True, unique=True)
     display_name = models.CharField(max_length=255)
     description = models.TextField(null=True)
@@ -146,10 +146,12 @@ class MailingList(models.Model):
             date of the first email.
         :param end_date: The cutoff for the newest thread.
         """
-        return self.threads.filter(
-            starting_email__date__lt=end_date,
-            date_active__gte=begin_date
-            ).order_by("-date_active")
+        return (self.threads.filter(
+            starting_email__date__lt=end_date, date_active__gte=begin_date)
+                .order_by("-date_active")
+                .select_related('starting_email')
+                .select_related('starting_email__sender')
+                )
 
     @property
     def recent_participants_count(self):
