@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2014-2021 by the Free Software Foundation, Inc.
+# Copyright (C) 2014-2022 by the Free Software Foundation, Inc.
 #
 # This file is part of HyperKitty.
 #
@@ -28,6 +28,7 @@ import re
 from django.conf import settings
 from django.contrib import messages
 from django.core.exceptions import SuspiciousOperation
+from django.db.models import Count
 from django.http import Http404, HttpResponse
 from django.shortcuts import get_object_or_404, redirect, render
 from django.template import loader
@@ -43,12 +44,15 @@ from hyperkitty.forms import AddTagForm, ReplyForm
 from hyperkitty.lib.utils import stripped_subject
 from hyperkitty.lib.view_helpers import (
     check_mlist_private, get_category_widget, get_months, get_posting_form)
-from hyperkitty.models import (
-    Favorite, LastView, MailingList, Tag, Tagging, Thread)
+from hyperkitty.models import Favorite, LastView, Tag, Tagging, Thread, Vote
 
 
 REPLY_RE = re.compile(r'^(re:\s*)*', re.IGNORECASE)
 
+#: GET request parameter that signifies the backend to respond with a page that
+#: doesn't require Javascript to function.
+NO_SCRIPT = 'noscript'
+
 
 def _get_thread_replies(request, thread, limit, offset=0):
     '''
@@ -63,15 +67,23 @@ def _get_thread_replies(request, thread, limit, offset=0):
     if sort_mode == "thread":
         sort_mode = "thread_order"
 
-    mlist = thread.mailinglist
+    mlist = request.mlist
     initial_subject = stripped_subject(mlist, thread.starting_email.subject)
-    emails = list(thread.emails.exclude(
-            pk=thread.starting_email.pk
-        ).order_by(sort_mode)[offset:offset+limit])
+    emails = list(thread
+                  .emails
+                  .exclude(pk=thread.starting_email.pk)
+                  .select_related('sender')
+                  .annotate(attachments_count=Count('attachments'))
+                  .order_by(sort_mode)[offset:offset+limit])
+    email_ids = [email.id for email in emails]
+    votes = dict(Vote
+                 .objects
+                 .filter(email_id__in=email_ids, user_id=request.user.id)
+                 .values_list('email_id', 'value'))
     for email in emails:
         # Extract all the votes for this message
         if request.user.is_authenticated:
-            email.myvote = email.votes.filter(user=request.user).first()
+            email.myvote = votes.get(email.id)
         else:
             email.myvote = None
         # Threading position
@@ -102,14 +114,26 @@ def _get_thread_replies(request, thread, limit, offset=0):
 @check_mlist_private
 def thread_index(request, mlist_fqdn, threadid, month=None, year=None):
     ''' Displays all the email for a given thread identifier '''
-    mlist = get_object_or_404(MailingList, name=mlist_fqdn)
+    # The check_mlist_private decorator sets the .mlist attribute
+    # on the request.
+    mlist = request.mlist
     thread = get_object_or_404(Thread, mailinglist=mlist, thread_id=threadid)
+    # Note(maxking): The query below fetches the thread along with it's
+    # starting_email in a single complex query, instead of two rather simple
+    # ones. We _could_ use the one below, but with simple tests in dev
+    # environment, there isn't a major speed bump that I am seeing, so I am
+    # going to keep it commented out.
+    #
+    # thread = Thread.objects.select_related('starting_email').get(
+    #      mailinglist=mlist, thread_id=threadid)
+
     starting_email = thread.starting_email
 
     sort_mode = request.GET.get("sort", "thread")
     if request.user.is_authenticated:
-        starting_email.myvote = starting_email.votes.filter(
-            user=request.user).first()
+        myvote = starting_email.votes.filter(user=request.user).first()
+        if myvote is not None:
+            starting_email.myvote = myvote.value
     else:
         starting_email.myvote = None
 
@@ -123,7 +147,7 @@ def thread_index(request, mlist_fqdn, threadid, month=None, year=None):
         fav_action = "rm"
 
     # Category
-    category, category_form = get_category_widget(request, thread.category)
+    # category, category_form = get_category_widget(request, thread.category)
 
     # Extract relative dates
     today = datetime.date.today()
@@ -168,6 +192,8 @@ def thread_index(request, mlist_fqdn, threadid, month=None, year=None):
         "title": _("This thread in gzipped mbox format"),
     }
 
+    is_noscript = NO_SCRIPT in request.GET
+
     context = {
         'mlist': mlist,
         'thread': thread,
@@ -182,17 +208,21 @@ def thread_index(request, mlist_fqdn, threadid, month=None, year=None):
         'fav_action': fav_action,
         'reply_form': get_posting_form(ReplyForm, request, mlist),
         'is_bot': is_bot,
+        'is_noscript': is_noscript,
         'num_comments': thread.emails_count - 1,
         'last_view': last_view,
         'unread_count': unread_count,
-        'category_form': category_form,
-        'category': category,
+        # XX(abraj): Commenting it out, since we don't need this. We aren't
+        # really using. At some point we can remove this but I am commenting.
+        #
+        # 'category_form': category_form,
+        # 'category': category,
         'export': export,
         'posting_enabled': getattr(
             settings, 'HYPERKITTY_ALLOW_WEB_POSTING', True),
     }
 
-    if is_bot:
+    if is_bot or is_noscript:
         # Don't rely on AJAX to load the replies
         # The limit is a safety measure, don't let a bot kill the DB
         context["replies"] = _get_thread_replies(request, thread, limit=1000)
@@ -206,9 +236,9 @@ def replies(request, mlist_fqdn, threadid):
     # chunk_size must be an even number, or the even/odd cycle will be broken.
     chunk_size = 6
     offset = int(request.GET.get("offset", "0"))
-    mlist = get_object_or_404(MailingList, name=mlist_fqdn)
-    thread = get_object_or_404(
-        Thread, mailinglist__name=mlist_fqdn, thread_id=threadid)
+    # This is set by the @check_mlist_private decorator.
+    mlist = request.mlist
+    thread = get_object_or_404(Thread, mailinglist=mlist, thread_id=threadid)
     # Last view
     last_view = request.GET.get("last_view")
     if last_view:
@@ -217,6 +247,7 @@ def replies(request, mlist_fqdn, threadid):
         except ValueError:
             last_view = None
     context = {
+        'mlist': mlist,
         'threadid': thread,
         'reply_form': get_posting_form(ReplyForm, request, mlist),
         'last_view': last_view,
@@ -248,7 +279,7 @@ def tags(request, mlist_fqdn, threadid):
         return HttpResponse('You must be logged in to add a tag',
                             content_type="text/plain", status=403)
     thread = get_object_or_404(
-        Thread, mailinglist__name=mlist_fqdn, thread_id=threadid)
+        Thread, mailinglist=request.mlist, thread_id=threadid)
 
     action = request.POST.get("action")
 
@@ -317,7 +348,7 @@ def favorite(request, mlist_fqdn, threadid):
                             content_type="text/plain", status=403)
 
     thread = get_object_or_404(
-        Thread, mailinglist__name=mlist_fqdn, thread_id=threadid)
+        Thread, mailinglist=request.mlist, thread_id=threadid)
     if request.POST["action"] == "add":
         Favorite.objects.get_or_create(thread=thread, user=request.user)
     elif request.POST["action"] == "rm":
@@ -336,7 +367,7 @@ def set_category(request, mlist_fqdn, threadid):
                             content_type="text/plain", status=403)
 
     thread = get_object_or_404(
-        Thread, mailinglist__name=mlist_fqdn, thread_id=threadid)
+        Thread, mailinglist=request.mlist, thread_id=threadid)
     category, category_form = get_category_widget(request)
     if not category and thread.category:
         thread.category = None
@@ -359,7 +390,7 @@ def reattach(request, mlist_fqdn, threadid):
     if not request.user.is_staff and not request.user.is_superuser:
         return HttpResponse('You must be a staff member to reattach a thread',
                             content_type="text/plain", status=403)
-    mlist = get_object_or_404(MailingList, name=mlist_fqdn)
+    mlist = request.mlist
     thread = get_object_or_404(Thread, mailinglist=mlist, thread_id=threadid)
     context = {
         'mlist': mlist,
@@ -407,7 +438,7 @@ def reattach(request, mlist_fqdn, threadid):
 
 @check_mlist_private
 def reattach_suggest(request, mlist_fqdn, threadid):
-    mlist = get_object_or_404(MailingList, name=mlist_fqdn)
+    mlist = request.mlist
     thread = get_object_or_404(Thread, mailinglist=mlist, thread_id=threadid)
 
     default_search_query = stripped_subject(
